from django.utils.translation import gettext_lazy as _

from wagtail_bynder import get_video_model

from .base import BaseBynderSyncCommand


class Command(BaseBynderSyncCommand):
    help = _(
        "Update stale Wagtail video library items to reflect recent asset updates in Bynder."
    )
    model = get_video_model()
    bynder_asset_type: str = "video"
    page_size: int = 200



from django.utils.translation import gettext_lazy as _
from wagtail.images import get_image_model

from .base import BaseBynderRefreshCommand


class Command(BaseBynderRefreshCommand):
    help = _(
        "Update ALL Wagtail image library items to reflect the latest data from Bynder."
    )
    model = get_image_model()



from django import template
from django.conf import settings


register = template.Library()


@register.simple_tag
def bynder_domain():
    return getattr(settings, "BYNDER_DOMAIN", "")


@register.simple_tag
def bynder_compactview_api_token():
    return getattr(settings, "BYNDER_COMPACTVIEW_API_TOKEN", "")



import logging
import math

from datetime import datetime
from mimetypes import guess_type
from typing import Any

from django.conf import settings
from django.db import models
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _
from wagtail.admin.panels import FieldPanel, MultiFieldPanel
from wagtail.documents.models import AbstractDocument, Document
from wagtail.images.models import AbstractImage, Image
from wagtail.models import Collection, CollectionMember
from wagtail.search import index

from wagtail_bynder import utils

from .exceptions import BynderAssetDataError


logger = logging.getLogger("wagtail.images")


class BynderAssetMixin(models.Model):
    # Fields relevant to the Bynder integration only
    bynder_id = models.CharField(
        verbose_name=_("Bynder asset ID"),
        max_length=36,
        blank=True,
        unique=True,
        editable=False,
        db_index=True,
        null=True,
    )
    bynder_last_modified = models.DateTimeField(
        null=True, editable=False, db_index=True
    )
    source_filename = models.CharField(
        verbose_name=_("source filename"),
        max_length=255,
        blank=True,
        editable=False,
    )
    original_filesize = models.IntegerField(
        verbose_name=_("original filesize"), editable=False, null=True
    )

    # Fields for broader use
    description = models.TextField(verbose_name=_("description"), blank=True)
    copyright = models.TextField(verbose_name=_("copyright info"), blank=True)
    is_archived = models.BooleanField(
        verbose_name=_("asset is archived"), default=False
    )
    is_limited_use = models.BooleanField(
        verbose_name=_("asset is marked as 'limited use'"),
        default=False,
    )
    is_public = models.BooleanField(
        verbose_name=_("asset is marked as public"), default=False
    )

    extra_admin_form_fields = (
        "description",
        "copyright",
        "is_archived",
        "is_limited_use",
        "is_public",
    )

    extra_search_fields = [
        index.SearchField("bynder_id", boost=3),
        index.SearchField("description"),
        index.SearchField("copyright"),
        index.AutocompleteField("bynder_id"),
        index.AutocompleteField("description"),
        index.AutocompleteField("copyright"),
    ]

    class Meta:
        abstract = True

    def is_up_to_date(self, asset_data: dict[str, Any]) -> bool:
        """
        Return a `bool` indicating whether if, based on the supplied `asset_data`,
        this object needs to be updated to reflect possible changes and resaved.
        """
        return (
            not self.bynder_last_modified
            or self.bynder_last_modified
            >= datetime.fromisoformat(asset_data["dateModified"])
        )

    def update_from_asset_data(
        self,
        asset_data: dict[str, Any],
        **kwargs,
    ) -> None:
        """
        Update this object (without saving) to reflect values in `asset_data`,
        which is a representation of the related asset from the Bynder API.

        NOTE: Although this base implementation does nothing with them currently,
        for compatibility reasons, ``**kwargs`` should always be accepted by all
        implementations of this method.
        """
        self.title = asset_data.get("name", self.title)
        self.copyright = asset_data.get("copyright", self.copyright)
        self.description = asset_data.get("description", self.description)
        self.collection = self.get_target_collection(asset_data)
        self.bynder_last_modified = asset_data["dateModified"]
        self.is_archived = bool(asset_data.get("archive", 0))
        self.is_limited_use = bool(asset_data.get("limited", 0))
        self.is_public = bool(asset_data.get("isPublic", 0))

    def get_target_collection(self, asset_data: dict[str, Any]) -> Collection:
        return utils.get_default_collection()


class BynderAssetWithFileMixin(BynderAssetMixin):
    extra_search_fields = BynderAssetMixin.extra_search_fields + [
        index.SearchField("file", boost=1),
        index.AutocompleteField("file"),
    ]

    class Meta:
        abstract = True

    @staticmethod
    def extract_file_source(asset_data: dict[str, Any]) -> str:
        raise NotImplementedError

    def update_from_asset_data(
        self, asset_data: dict[str, Any], *, force_download: bool = False, **kwargs
    ) -> None:
        """
        Overrides ``BynderAssetMixin.update_from_asset_data()`` to explicitly
        handle the ``force_download`` option that can be provided by management
        commands, and to initiate downloading of the source file when it has
        changed in some way.
        """
        super().update_from_asset_data(asset_data, **kwargs)
        if force_download or not self.file or self.asset_file_has_changed(asset_data):
            self.update_file(asset_data)

    def asset_file_has_changed(self, asset_data: dict[str, Any]) -> bool:
        source_url = self.extract_file_source(asset_data)
        filename = utils.filename_from_url(source_url)
        return (
            self.source_filename != filename
            or self.original_filesize is None
            or self.original_filesize != int(asset_data["fileSize"])
        )

    def update_file(self, asset_data: dict[str, Any]) -> None:
        source_url = self.extract_file_source(asset_data)
        self.file = utils.download_asset(source_url)

        # Used to trigger additional updates on save()
        self._file_changed = True

        # Update supplementary field values
        self.source_filename = utils.filename_from_url(source_url)
        self.original_filesize = int(asset_data["fileSize"])


class BynderSyncedImage(BynderAssetWithFileMixin, AbstractImage):
    admin_form_fields = (
        Image.admin_form_fields + BynderAssetMixin.extra_admin_form_fields
    )

    original_width = models.IntegerField(
        verbose_name=_("original width"), null=True, editable=False
    )
    original_height = models.IntegerField(
        verbose_name=_("original height"), null=True, editable=False
    )

    search_fields = (
        AbstractImage.search_fields + BynderAssetWithFileMixin.extra_search_fields
    )

    class Meta(AbstractImage.Meta):
        abstract = True

    def save(self, *args, **kwargs):
        if getattr(self, "_file_changed", False):
            self._set_image_file_metadata()
        if self.pk and (
            getattr(self, "_file_changed", False)
            or getattr(self, "_focal_point_changed", False)
        ):
            # wagtail.images.forms.BaseImageForm usually takes care of this when
            # updating via the UI. But, if updating objects directly, we must
            # delete stale renditions ourselves.
            self.renditions.all().delete()
        super().save(*args, **kwargs)

    def update_from_asset_data(
        self, asset_data: dict[str, Any], *, force_download: bool = False, **kwargs
    ) -> None:
        """
        Overrides ``BynderAssetWithFileMixin.update_from_asset_data()`` to
        handle conversion of focal points to focal areas.
        """

        # Update the file and other field values without saving the changes
        super().update_from_asset_data(
            asset_data, force_download=force_download, **kwargs
        )

        # Update the focal area if a focus point is set
        current_focal_point = self.get_focal_point()
        self._focal_point_changed = False
        focus_point = asset_data.get("activeOriginalFocusPoint")
        if focus_point:
            self.set_focal_area_from_focus_point(
                int(focus_point["x"]),
                int(focus_point["y"]),
                int(asset_data["height"]),
                int(asset_data["width"]),
            )
            self._focal_point_changed = (
                current_focal_point is None
                or self.get_focal_point() != current_focal_point
            )

    def asset_file_has_changed(self, asset_data: dict[str, Any]) -> bool:
        return (
            super().asset_file_has_changed(asset_data)
            or (self.original_height or 0) != int(asset_data["height"])
            or (self.original_width or 0) != int(asset_data["width"])
        )

    def update_file(self, asset_data: dict[str, Any]) -> None:
        self.original_width = int(asset_data["width"])
        self.original_height = int(asset_data["height"])
        return super().update_file(asset_data)

    def set_focal_area_from_focus_point(
        self, x: int, y: int, original_height: int, original_width: int
    ) -> None:
        """
        Using the provided focus point coordinates, generate a
        2D focal area for the downloaded image.
        """
        if x < 0 or y < 0 or x > original_width or y > original_height:
            raise ValueError(
                "Focus point coordinates must be inside the original image bounds"
            )

        # Scale the coordinates to reflect the dimensions of the image that
        # was actually downloaded
        if self.height != original_height:
            scale = original_height / self.height
            x = math.floor(x / scale)
            y = math.floor(y / scale)

        # Set the centre point
        self.focal_point_x = x
        self.focal_point_y = y

        # Draw a rectangle around the centre point
        # For the width, span outwards until we hit the left or right bounds
        rect_width = min(x, self.width - x) * 2
        # Restrict rectangle width to 40% of the image height
        rect_width = min(rect_width, math.floor(self.width * 0.4))

        # For the height, span outwards until we hit the top or bottom bounds
        rect_height = min(y, self.height - y) * 2
        # Restrict rectangle height to 40% of the image height
        rect_height = min(rect_height, math.floor(self.height * 0.4))

        # Use the shortest side to make a square
        width = min(rect_width, rect_height)
        self.focal_point_width = width
        self.focal_point_height = width

    @staticmethod
    def extract_file_source(asset_data: dict[str, Any]) -> str:
        # For images, we store and use the source derivative filename,
        # because the 'original' isn't always present
        asset_id = asset_data["id"]
        key = getattr(settings, "BYNDER_IMAGE_SOURCE_THUMBNAIL_NAME", "WagtailSource")
        thumbnails = asset_data["thumbnails"]
        try:
            return thumbnails[key]
        except KeyError as e:
            raise BynderAssetDataError(
                f"The '{key}' derivative is missing from 'thumbnails' for image asset '{asset_id}'. "
                "You might need to update the 'BYNDER_IMAGE_SOURCE_THUMBNAIL_NAME' setting value to reflect "
                "derivative names used in your Bynder instance. The available derivatives for this asset "
                f"are: {list(thumbnails.keys())}"
            ) from e


class BynderSyncedDocument(BynderAssetWithFileMixin, AbstractDocument):
    admin_form_fields = (
        Document.admin_form_fields + BynderAssetMixin.extra_admin_form_fields
    )

    search_fields = (
        AbstractDocument.search_fields + BynderAssetWithFileMixin.extra_search_fields
    )

    class Meta(AbstractDocument.Meta):
        abstract = True

    def save(self, *args, **kwargs):
        if getattr(self, "_file_changed", False):
            self._set_document_file_metadata()
        super().save(*args, **kwargs)

    @staticmethod
    def extract_file_source(asset_data: dict[str, Any]) -> str:
        asset_id = asset_data["id"]
        try:
            return asset_data["original"]
        except KeyError as e:
            raise BynderAssetDataError(
                f"'original' is missing from the API representation for document asset '{asset_id}'. "
                "This is likely because the asset is marked as 'private' in Bynder. Wagtail needs the "
                "'original' asset URL in order to download and save its own copy."
            ) from e


class BynderSyncedVideo(
    BynderAssetMixin, CollectionMember, index.Indexed, models.Model
):
    title = models.CharField(max_length=255, verbose_name=_("title"))
    created_at = models.DateTimeField(
        verbose_name=_("created at"), auto_now_add=True, db_index=True
    )
    updated_at = models.DateTimeField(
        verbose_name=_("last updated at"), auto_now=True, db_index=True
    )
    original_width = models.IntegerField(
        verbose_name=_("original width"), null=True, editable=False
    )
    original_height = models.IntegerField(
        verbose_name=_("original height"), null=True, editable=False
    )
    primary_source_url = models.URLField(
        verbose_name=_("primary source URL"),
        help_text=_(
            "A derivative using a WebM container using the VP9 codec for video and the Opus codec "
            "for audio. These are all open, royalty-free formats which are generally "
            "well-supported, although only in quite recent browsers, which is why a fallback is a "
            "good idea."
        ),
    )
    fallback_source_url = models.URLField(
        blank=True,
        verbose_name=_("fallback source URL"),
        help_text=(
            "A derivative using an MP4 container and the AVC (H.264) video codec, ideally with "
            "AAC as your audio codec. This is because the MP4 container with AVC and AAC codecs "
            "within is a broadly-supported combination—by every major browser, in fact—and the "
            "quality is typically good for most use cases."
        ),
    )
    poster_image_url = models.URLField(verbose_name=_("poster image URL"))

    search_fields = [
        index.SearchField("title", boost=3),
        index.AutocompleteField("title"),
        index.SearchField("primary_source_url", boost=1),
        index.AutocompleteField("primary_source_url"),
    ] + BynderAssetMixin.extra_search_fields

    panels = [
        FieldPanel("title"),
        FieldPanel("description"),
        FieldPanel("primary_source_url"),
        FieldPanel("fallback_source_url"),
        FieldPanel("poster_image_url"),
        FieldPanel("copyright", read_only=True),
        MultiFieldPanel(
            heading=_("Status flags"),
            children=[
                FieldPanel("is_archived", read_only=True),
                FieldPanel("is_limited_use", read_only=True),
                FieldPanel("is_public", read_only=True),
            ],
        ),
        MultiFieldPanel(
            heading=_("Further information"),
            children=[
                FieldPanel("original_filesize", read_only=True),
                FieldPanel("original_height", read_only=True),
                FieldPanel("original_width", read_only=True),
            ],
        ),
    ]

    class Meta:
        abstract = True
        verbose_name = _("video")
        verbose_name_plural = _("videos")

    def __str__(self):
        return self.title

    @cached_property
    def primary_source_mimetype(self) -> str:
        if not self.primary_source_url:
            return ""
        return guess_type(self.primary_source_url)[0]

    @cached_property
    def fallback_source_mimetype(self) -> str:
        if not self.fallback_source_url:
            return ""
        return guess_type(self.fallback_source_url)[0]

    def update_from_asset_data(
        self,
        asset_data: dict[str, Any],
        **kwargs,
    ) -> None:
        """
        Overrides ``BynderAssetMixin.update_from_asset_data()`` to handle
        setting of video-specific field values.
        """

        primary_derivative_name = getattr(
            settings, "BYNDER_VIDEO_PRIMARY_DERIVATIVE_NAME", "WebPrimary"
        )
        fallback_derivative_name = getattr(
            settings, "BYNDER_VIDEO_FALLBACK_DERIVATIVE_NAME", "WebFallback"
        )
        poster_image_derivative_name = getattr(
            settings, "BYNDER_VIDEO_POSTER_IMAGE_DERIVATIVE_NAME", "webimage"
        )

        derivatives = {v.split("/")[-2]: v for v in asset_data["videoPreviewURLs"]}
        try:
            self.primary_source_url = derivatives[primary_derivative_name]
        except KeyError as e:
            raise BynderAssetDataError(
                "'videoPreviewURLs' does not contain a URL matching the derivative name "
                f"'{primary_derivative_name}'. You might need to update the "
                "'BYNDER_VIDEO_PRIMARY_DERIVATIVE_NAME' setting value to reflect the derivative "
                "names set by Bynder for your instance. The available derivatives for this "
                f"asset are: {list(derivatives.keys())}"
            ) from e
        else:
            self.source_filename = utils.filename_from_url(self.primary_source_url)

        self.fallback_source_url = derivatives.get(fallback_derivative_name)

        thumbnails = asset_data["thumbnails"]
        try:
            self.poster_image_url = thumbnails[poster_image_derivative_name]
        except KeyError as e:
            raise BynderAssetDataError(
                f"The '{poster_image_derivative_name}' derivative is missing from 'thumbnails' for "
                f"video asset '{self.bynder_id}'. You might need to update the "
                "'BYNDER_VIDEO_POSTER_IMAGE_DERIVATIVE_NAME' setting value to reflect the "
                "derivative names set by Bynder for you instance. The available derivative names "
                f"for this asset are: {list(thumbnails.keys())}"
            ) from e

        self.original_filesize = int(asset_data["fileSize"])
        self.original_width = int(asset_data["width"])
        self.original_height = int(asset_data["height"])

        super().update_from_asset_data(asset_data, **kwargs)



from django.utils.translation import gettext_lazy as _

from wagtail_bynder import get_video_model

from .base import BaseBynderRefreshCommand


class Command(BaseBynderRefreshCommand):
    help = _(
        "Update ALL Wagtail video library items to reflect the latest data from Bynder."
    )
    model = get_video_model()



from typing import TYPE_CHECKING

from django.conf import settings
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _
from wagtail.admin.views.generic.chooser import ChooseView
from wagtail.snippets.views.chooser import SnippetChooserViewSet, SnippetChosenView
from wagtail.snippets.views.snippets import EditView, SnippetViewSet

from wagtail_bynder import get_video_model

from .mixins import BynderAssetCopyMixin, RedirectToBynderMixin


if TYPE_CHECKING:
    from django.http import HttpRequest, JsonResponse


class VideoEditView(EditView, RedirectToBynderMixin):
    model = get_video_model()
    pk_url_kwarg = "pk"


class VideoChooseView(ChooseView):
    filter_form_class = None
    page_title = _("Choose")
    template_name = "wagtailadmin/chooser/chooser-bynder.html"

    @property
    def page_subtitle(self):
        return self.model._meta.verbose_name


class VideoChosenView(SnippetChosenView, BynderAssetCopyMixin):
    def get(self, request: "HttpRequest", pk: str) -> "JsonResponse":
        try:
            obj = self.model.objects.get(bynder_id=pk)
        except self.model.DoesNotExist:
            obj = self.create_object(pk)
        else:
            if getattr(settings, "BYNDER_SYNC_EXISTING_VIDEOS_ON_CHOOSE", False):
                self.update_object(pk, obj)
        return self.get_chosen_response(obj)


class VideoChooserViewSet(SnippetChooserViewSet):
    choose_one_text = _("Choose a video")
    choose_another_text = _("Choose another video")
    edit_item_text = _("Edit this video")

    choose_view_class = VideoChooseView
    chosen_view_class = VideoChosenView

    @cached_property
    def widget_class(self):
        from wagtail_bynder.widgets import AdminVideoChooser

        return AdminVideoChooser(icon=self.icon)


class VideoViewSet(SnippetViewSet):
    icon = "media"
    edit_view_class = VideoEditView
    chooser_viewset_class = VideoChooserViewSet
    chooser_per_page = 50
    list_display = ["title", "bynder_id", "created_at", "updated_at"]

    def __init__(self, *args, **kwargs):
        self.model = get_video_model()
        super().__init__(*args, **kwargs)



from django.utils.translation import gettext_lazy as _
from wagtail.documents import get_document_model

from .base import BaseBynderSyncCommand


class Command(BaseBynderSyncCommand):
    help = _(
        "Update stale Wagtail document library items to reflect recent asset updates in Bynder."
    )
    model = get_document_model()
    bynder_asset_type: str = "document"
    page_size: int = 200



from collections.abc import Iterable
from datetime import datetime
from typing import TYPE_CHECKING, Any

from django.core.management.base import BaseCommand
from django.db.models.base import ModelBase
from django.db.models.query import Q
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from requests import HTTPError

from wagtail_bynder.models import BynderAssetMixin
from wagtail_bynder.utils import get_bynder_client


if TYPE_CHECKING:
    from django.db.models.query import QuerySet


class BaseModelCommand(BaseCommand):
    model: ModelBase | None = None

    def get_queryset(self) -> "QuerySet":
        return self.model.objects.all()  # type: ignore[attr-defined]


class BaseBynderSyncCommand(BaseModelCommand):
    bynder_asset_type: str = ""
    page_size: int = 200

    def add_arguments(self, parser):
        parser.add_argument(
            "--minutes",
            type=int,
            help=_(
                "The number of minutes into the past to look for asset "
                "modifications."
            ),
        )
        parser.add_argument(
            "--hours",
            type=int,
            help=_(
                "The number of hours into the past to look for asset "
                "modifications (takes precedence over 'minutes')"
            ),
        )
        parser.add_argument(
            "--days",
            type=int,
            help=_(
                "The number of days in the past to look for asset "
                "modifications (takes precedence over 'hours' and 'minutes')"
            ),
        )

    def handle(self, *args, **options):
        # Default timespan to 1 day (1440 minutes)
        minutes = options.get("minutes")
        hours = options.get("hours")
        days = options.get("days")
        if days:
            timespan = timezone.timedelta(days=days)
            timespan_desc = f"{days} day(s)"
        elif hours:
            timespan = timezone.timedelta(hours=hours)
            timespan_desc = f"{hours} hour(s)"
        elif minutes:
            timespan = timezone.timedelta(minutes=minutes)
            timespan_desc = f"{minutes} minute(s)"
        else:
            timespan = timezone.timedelta(days=1)
            timespan_desc = "1 day"

        self.date_modified_from = datetime.utcnow() - timespan

        self.stdout.write(
            f"Looking for {self.bynder_asset_type or 'all'} assets modified within the last {timespan_desc}"
        )

        self.batch_count = 1
        self.bynder_client = get_bynder_client()
        asset_dict: dict[str, dict[str, Any]] = {}

        for asset in self.get_assets():
            # Gather asset details into a large dict, using the 'id' as the key
            asset_dict[asset["id"]] = asset
            # Process the gathered assets once the batch reaches a certain size
            if len(asset_dict) == self.page_size:
                self.process_batch(asset_dict)
                # Clear this batch to start another
                asset_dict.clear()

        # Process any remaining assets
        if asset_dict:
            self.process_batch(asset_dict)

    def get_assets(self) -> Iterable[dict[str, Any]]:
        """
        A generator method that yields all relevant Bynder assets, one at a time.
        It silently uses pagination to ensure all possible assets are returned.
        """
        page = 1
        while True:
            query = {
                # Datetimes must be supplied in ISO 8601 format without microseconds. See:
                # https://bynder.docs.apiary.io/#reference/assets/asset-operations/retrieve-assets
                "dateModified": self.date_modified_from.strftime("%Y-%m-%dT%H:%M:%SZ"),
                "orderBy": "dateModified desc",
                "page": page,
                "limit": self.page_size,
            }
            if self.bynder_asset_type:
                query["type"] = self.bynder_asset_type
            results = self.bynder_client.asset_bank_client.media_list(query)
            if not results:
                break
            yield from results
            if len(results) < self.page_size:
                break
            page += 1
        return

    def process_batch(self, assets: dict[str, dict[str, Any]]) -> None:
        """
        Identifies and updates (where needed) model objects to reflect changes
        in the supplied 'batch' of Bynder assets.
        """
        self.stdout.write(
            f"Processing batch {self.batch_count} ({len(assets)} assets)..."
        )
        self.batch_count += 1

        stale = self.get_stale_objects(assets)
        self.stdout.write(f"{len(stale)} stale objects were found for this batch.")
        for obj in stale:
            data = assets[obj.bynder_id]
            self.update_object(obj, data)

    def get_stale_objects(self, assets: dict[str, dict[str, Any]]) -> "QuerySet":
        """
        Return a queryset of model instances that represent items in the supplied
        batch of assets, and are out-of-sync with the data in Bynder (and
        therefore should be updated).
        """
        q = Q()
        for id, asset in assets.items():
            # excluding anything where 'bynder_last_modified' value is equal to
            # that from Bynder (which means it is already up-to-date)
            q |= Q(bynder_id=id, bynder_last_modified__lt=asset["dateModified"])
        return self.get_queryset().filter(q)

    def update_object(self, obj: BynderAssetMixin, asset_data: dict[str, Any]) -> None:
        self.stdout.write("\n")
        self.stdout.write(f"Updating object for asset '{asset_data['id']}'")
        if obj.bynder_last_modified:
            time_diff = (
                datetime.fromisoformat(asset_data["dateModified"])
                - obj.bynder_last_modified
            )
            self.stdout.write(f"{repr(obj)} is behind by: {time_diff}")
        self.stdout.write("The latest data from Bynder is:")
        for key, value in asset_data.items():
            self.stdout.write(f"  {key}: {value}")
        self.stdout.write("-" * 80)

        obj.update_from_asset_data(asset_data)
        obj.save()


class BaseBynderRefreshCommand(BaseModelCommand):
    def add_arguments(self, parser):
        parser.add_argument(
            "--from",
            type=int,
            help=_(
                "Only update items with a 'pk' value greater than or equal to this value."
            ),
        )
        parser.add_argument(
            "--force-download",
            action="store_true",
            help=_(
                "Force redownloading and updating of files, regardless of whether they have changed."
            ),
        )
        parser.add_argument(
            "--delete-not-recognised",
            action="store_true",
            help=_(
                "Delete local objects with a 'bynder_id' that is no longer recognised by Bynder"
            ),
        )

    def handle(self, *args, **options):
        self.batch_count = 0
        self.bynder_client = get_bynder_client()
        self.force_download = options["force_download"]
        self.from_pk = options["from"]
        self.delete_not_recognised = options["delete_not_recognised"]
        unrecognised_asset_ids = []

        for obj in self.get_queryset():
            try:
                asset_data = self.bynder_client.asset_bank_client.media_info(
                    obj.bynder_id
                )
            except HTTPError as e:
                if e.response.status_code == 404:
                    self.stdout.write(
                        f"Asset ID '{obj.bynder_id}' was not recognized by Bynder\n"
                    )
                    unrecognised_asset_ids.append(obj.bynder_id)
                    continue
                else:
                    raise e
            else:
                self.stdout.write(
                    f"Asset with ID '{asset_data['id']}' was fetched successfully\n"
                )
                self.update_object(obj, asset_data)

        self.stdout.write(
            f"During this run, {len(unrecognised_asset_ids)} asset id(s) were not recognised by Bynder"
        )
        if unrecognised_asset_ids:
            self.stdout.write("\n".join(unrecognised_asset_ids))
            if self.delete_not_recognised:
                for obj in self.get_queryset().filter(
                    bynder_id__in=unrecognised_asset_ids
                ):
                    obj.delete()
                self.stdout.write(
                    f"All local {self.model._meta.label} objects using these IDs have been deleted."  # type: ignore[attr-defined]
                )

    def get_queryset(self) -> "QuerySet":
        queryset = super().get_queryset().exclude(bynder_id__isnull=True).order_by("pk")
        if self.from_pk:
            return queryset.filter(pk__gte=self.from_pk)
        return queryset

    def update_object(self, obj: BynderAssetMixin, asset_data: dict[str, Any]) -> None:
        self.stdout.write(
            f"Updating <{self.model._meta.label}: pk='{obj.pk}' title='{obj.title}'>"  # type: ignore[attr-defined]
        )
        obj.update_from_asset_data(asset_data, force_download=self.force_download)
        obj.save()



from django.conf import settings
from django.utils.deprecation import MiddlewareMixin
from wagtail.admin.auth import require_admin_access

from wagtail_bynder.views import document as document_views
from wagtail_bynder.views import image as image_views


class PatchWagtailURLsMiddleware(MiddlewareMixin):
    @classmethod
    def get_overrides(cls) -> dict[str, callable]:
        overrides = {}
        if getattr(settings, "BYNDER_DOMAIN", ""):
            overrides.update(
                {
                    "wagtailimages_chooser:choose": image_views.ImageChooseView.as_view(),
                    "wagtailimages_chooser:chosen": image_views.ImageChosenView.as_view(),
                    "wagtaildocs_chooser:choose": document_views.DocumentChooseView.as_view(),
                    "wagtaildocs_chooser:chosen": document_views.DocumentChosenView.as_view(),
                }
            )
            if getattr(settings, "BYNDER_DISABLE_WAGTAIL_EDITING_FOR_ASSETS", False):
                overrides.update(
                    {
                        "wagtailimages:edit": image_views.ImageEditView.as_view(),
                        "wagtailimages:delete": image_views.ImageDeleteView.as_view(),
                        "wagtaildocs:edit": document_views.DocumentEditView.as_view(),
                        "wagtaildocs:delete": document_views.DocumentDeleteView.as_view(),
                    }
                )
        return overrides

    def process_view(self, request, view_func, view_args, view_kwargs):
        overrides = self.get_overrides()
        replacement_view = overrides.get(request.resolver_match.view_name)
        if replacement_view:
            view = require_admin_access(replacement_view)
            return view(request, *view_args, **view_kwargs)



from django import forms
from django.urls import reverse
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _
from wagtail.admin.staticfiles import versioned_static
from wagtail.admin.widgets import BaseChooser, BaseChooserAdapter
from wagtail.telepath import register

from wagtail_bynder import get_video_model


class AdminVideoChooser(BaseChooser):
    choose_one_text = _("Choose a video")
    choose_another_text = _("Change video")
    link_to_chosen_text = _("Edit this video")
    chooser_modal_url_name = "bynder_video_chooser:choose"
    icon = "media"
    classname = "video-chooser"
    js_constructor = "VideoChooser"

    def __init__(self, **kwargs):
        self.model = get_video_model()
        super().__init__(**kwargs)

    def get_chooser_modal_url(self):
        return reverse(
            self.model.snippet_viewset.chooser_viewset.get_url_name("choose")
        )

    @property
    def media(self):
        return forms.Media(
            js=[
                versioned_static("wagtailsnippets/js/snippet-chooser.js"),
                versioned_static("bynder/js/video-chooser.js"),
            ]
        )


class VideoChooserAdapter(BaseChooserAdapter):
    js_constructor = "wagtail_bynder.widgets.VideoChooser"

    @cached_property
    def media(self):
        return forms.Media(
            js=[
                versioned_static("bynder/js/video-chooser-telepath.js"),
            ]
        )


register(VideoChooserAdapter(), AdminVideoChooser)
